﻿using Syncfusion.DataSource;
using Syncfusion.ListView.XForms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Emit;
using System.Text;

using Xamarin.Forms;

namespace Grouping
{
    public class GroupingPageCS : ContentPage
    {
        #region Fields

        ContactsViewModel viewModel;
        SfListView listView;

        #endregion

        #region Constructor

        public GroupingPageCS()
        {
            Content = InitializeView();
            Title = "SfListView Grouping Sample";
        }

        #endregion

        #region Private Methods

        private Grid InitializeView()
        {
            var grid = new Grid();
            var rowdefinition0 = new RowDefinition { Height = 50 };
            var rowdefinition1 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            grid.RowDefinitions.Add(rowdefinition0);
            grid.RowDefinitions.Add(rowdefinition1);

            var button = new Button() { BackgroundColor = Color.Teal, BorderColor = Color.Gray, TextColor = Color.White, Text = "Group" };
            button.Clicked += Button_Clicked;

            InitializeListView();

            grid.Children.Add(button);
            grid.Children.Add(listView, 0, 1);

            return grid;
        }

        private void InitializeListView()
        {
            listView = new SfListView();
            viewModel = new ContactsViewModel();
            listView.ItemsSource = viewModel.Contactsinfo;
            listView.ItemTemplate = new DataTemplate(() => { return CreateItemTemplate(); });
            listView.ItemSize = 70;
            listView.GroupHeaderSize = 60;
            listView.IsStickyGroupHeader = true;
            listView.AllowGroupExpandCollapse = true;
            listView.ItemSpacing = new Thickness(0, 0, 5, 0);
            listView.IsStickyHeader = true;
            listView.GroupExpanding += ListView_GroupExpanding;
            listView.GroupCollapsing += ListView_GroupCollapsing;
            listView.GroupHeaderTemplate = new DataTemplate(() =>
            {
                var grid = new Grid { BackgroundColor = Color.FromHex("#E4E4E4") };
                var label = new Label
                {
                    FontAttributes = FontAttributes.Bold,
                    FontSize = 22,
                    VerticalOptions = LayoutOptions.Center,
                    HorizontalOptions = LayoutOptions.Start,
                    Margin = new Thickness(20, 0, 0, 0),
                };
                label.SetBinding(Label.TextProperty, new Binding("Key"));
                grid.Children.Add(label);
                return grid;
            });
            listView.DataSource.SortDescriptors.Add(new SortDescriptor { PropertyName = "ContactName", Direction = ListSortDirection.Ascending });
            //Applying Custom Grouping
            listView.DataSource.GroupDescriptors.Add(new GroupDescriptor()
            {
                PropertyName = "ContactName"
            });

        }

        private Grid CreateItemTemplate()
        {
            var gridView = new Grid();
            gridView.RowSpacing = 1;
            var row0 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            var row1 = new RowDefinition { Height = 1 };
            gridView.RowDefinitions.Add(row0);
            gridView.RowDefinitions.Add(row1);

            var grid = new Grid();

            var column0 = new ColumnDefinition { Width = 50 };
            var column1 = new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) };
            var column2 = new ColumnDefinition { Width = 70 };
            grid.ColumnDefinitions.Add(column0);
            grid.ColumnDefinitions.Add(column1);
            grid.ColumnDefinitions.Add(column2);

            var contactimage = new Image();
            contactimage.SetBinding(Image.SourceProperty, new Binding("ContactImage"));
            contactimage.HeightRequest = 50;
            contactimage.VerticalOptions = LayoutOptions.Center;
            contactimage.HorizontalOptions = LayoutOptions.Center;

            var gridview = new Grid();

            gridview.RowSpacing = 1;
            gridview.Padding = new Thickness(10, 0, 0, 0);
            gridview.VerticalOptions = LayoutOptions.Center;
            var rowdefinition0 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            var rowdefinition1 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            gridview.RowDefinitions.Add(rowdefinition0);
            gridview.RowDefinitions.Add(rowdefinition1);

            var contactName = new Label();
            contactName.SetBinding(Label.TextProperty, new Binding("ContactName"));
            contactName.LineBreakMode = LineBreakMode.NoWrap;
            contactName.FontAttributes = FontAttributes.Bold;
            contactName.TextColor = Color.FromHex("#474747");
            contactName.FontSize = 20;

            var contactNumber = new Label();
            contactNumber.FontAttributes = FontAttributes.None;
            contactNumber.FontSize = 18;
            contactNumber.LineBreakMode = LineBreakMode.NoWrap;
            contactNumber.SetBinding(Label.TextProperty, new Binding("ContactNumber"));
            contactNumber.TextColor = Color.FromHex("#474747");

            var contactType = new Label();
            contactType.Margin = new Thickness(5);
            contactType.FontAttributes = FontAttributes.None;
            contactType.LineBreakMode = LineBreakMode.NoWrap;
            contactType.VerticalOptions = LayoutOptions.End;
            contactType.VerticalTextAlignment = TextAlignment.End;
            contactType.HorizontalOptions = LayoutOptions.End;
            contactType.SetBinding(Label.TextProperty, new Binding("ContactType"));
            contactType.TextColor = Color.FromHex("#474747");
            contactType.FontSize = 12;

            var stackLayout = new StackLayout();
            stackLayout.HeightRequest = 1;
            stackLayout.BackgroundColor = Color.Gray;

            gridview.Children.Add(contactName, 0, 0);
            gridview.Children.Add(contactNumber, 0, 1);
            gridview.Children.Add(contactType, 1, 0);

            grid.Children.Add(contactimage);
            grid.Children.Add(gridview, 1, 0);

            gridView.Children.Add(grid);
            gridView.Children.Add(stackLayout, 0, 1);

            return gridView;
        }

        #endregion

        #region CallBacks

        private void Button_Clicked(object sender, EventArgs e)
        {
            //Applying Grouping when group button is clicked
            listView.DataSource.GroupDescriptors.Clear();
            listView.DataSource.GroupDescriptors.Add(new GroupDescriptor()
            {
                PropertyName = "ContactName",
                KeySelector = (object obj1) =>
                {
                    var item = (obj1 as Contacts);
                    return item.ContactName[0].ToString();
                },
                Comparer = new CustomGroupComparer()
            });
        }

        private void ListView_GroupExpanding(object sender, GroupExpandCollapseChangingEventArgs e)
        {
            if (e.Groups[0] == listView.DataSource.Groups[0])
                e.Cancel = true;
        }

        private void ListView_GroupCollapsing(object sender, GroupExpandCollapseChangingEventArgs e)
        {
            if (e.Groups[0] == listView.DataSource.Groups[0])
                e.Cancel = true;
        }

        #endregion
    }
}
